@@ -310,6 +310,8 @@ namespace shevek
 		{
 		default: // One word
 			s >> ret;
+			if (ret.size () == 0)
+				return false;
 			break;
 		case 'l': // One line
 			std::getline (s, line);
@@ -340,10 +342,11 @@ namespace shevek
 			return true;
 		}
 		std::streampos p = s.tellg ();
-		if (!s || p < 0)
-			return false;
-		pos.top () += p;
-		return s;
+		if (p >= 0)
+			pos.top () += p;
+		else
+			pos.top () = data.size ();
+		return true;
 	}
 
 	static int make_base (gunichar code)
@@ -381,9 +384,9 @@ namespace shevek
 			int &ret, Glib::ustring::size_type &inpos)
 	{
 		int base = make_base (format[inpos++]);
-		Glib::ustring d = data.substr (pos.top ());
-		if (read_char (d, pos.top (), ret))
+		if (read_char (data, pos.top (), ret))
 			return true;
+		Glib::ustring d = data.substr (pos.top ());
 		char *end;
 		char const *str = d.c_str ();
 		ret = strtol (str, &end, base);
@@ -398,13 +401,13 @@ namespace shevek
 			unsigned &ret, Glib::ustring::size_type &inpos)
 	{
 		int base = make_base (format[inpos++]);
-		Glib::ustring d = data.substr (pos.top ());
 		int r;
-		if (read_char (d, pos.top (), r))
+		if (read_char (data, pos.top (), r))
 		{
 			ret = r;
 			return true;
 		}
+		Glib::ustring d = data.substr (pos.top ());
 		char *end;
 		char const *str = d.c_str ();
 		ret = strtoul (str, &end, base);
@@ -796,6 +799,8 @@ namespace shevek
 		{
 		default: // One word
 			s >> ret;
+			if (ret.size () == 0)
+				return false;
 			break;
 		case 'l': // One line
 			std::getline (s, ret);
@@ -825,10 +830,11 @@ namespace shevek
 			return true;
 		}
 		std::streampos p = s.tellg ();
-		if (!s || p < 0)
-			return false;
-		pos.top () += p;
-		return s;
+		if (p >= 0)
+			pos.top () += p;
+		else
+			pos.top () = data.size ();
+		return true;
 	}
 
 	static bool read_char (std::string const &data,
@@ -850,9 +856,9 @@ namespace shevek
 			int &ret, std::string::size_type &inpos)
 	{
 		int base = make_base (format[inpos++]);
-		std::string d = data.substr (pos.top ());
-		if (read_char (d, pos.top (), ret))
+		if (read_char (data, pos.top (), ret))
 			return true;
+		std::string d = data.substr (pos.top ());
 		char *end;
 		char const *str = d.c_str ();
 		ret = strtol (str, &end, base);
@@ -867,13 +873,13 @@ namespace shevek
 			unsigned &ret, std::string::size_type &inpos)
 	{
 		int base = make_base (format[inpos++]);
-		std::string d = data.substr (pos.top ());
 		int r;
-		if (read_char (d, pos.top (), r))
+		if (read_char (data, pos.top (), r))
 		{
 			ret = r;
 			return true;
 		}
+		std::string d = data.substr (pos.top ());
 		char *end;
 		char const *str = d.c_str ();
 		ret = strtoul (str, &end, base);
